Abstract
-----------------
The classification scheme previously published by the authors, and called WT-EMP,
introduces spatial information in the classification process by means of an 
Extended Morphological Profile (EMP) that is created from features extracted by 
wavelets. In addition, the hyperspectral image is denoised in the 2-D spatial 
domain, also using wavelets and it is joined to the EMP via a stacked vector. 

In this paper, the scheme is improved achieving two goals. The first one is to
reduce the classification time while preserving the accuracy of the 
classification by using ELM instead of SVM. The second one is to improve the 
accuracy results by performing not only a 2-D denoising for every spectral
band, but also a previous additional 1-D spectral signature denoising applied 
to each pixel vector of the image. For each denoising the image is transformed 
by applying a 1-D or 2-D wavelet transform, and then a NeighShrink thresholding
is applied.

[1] Pablo Quesada-Barriuso, Dora B. Heras and Francisco Argüello, "Wavelet-based
Denoising in the Classification of Remote Sensing Hyperspectral Images", SPIE 
Remote Sensing Conference (2016). 

WTSS_EMP_Scheme.m
-----------------
WTSS-EMP scheme based on wavelets, EMP and ELM.

Example of use in MATLAB:

    load /datos/pablo.quesada/Data/Pavia_University/paviau.mat
    x = scale_data(x, 0, 1);
    [ x t ] = WTSS_EMP_Scheme( x );
    
    load ./tt_samples/paviau_train_samples_as_review2013.mat
    k = 1; % to use a different set of training samples changes the k values

    % prepare the data one pixel-vector per row
    x = row_major(x);
    % load the reference map, also with one pixel per row
    load /datos/pablo.quesada/Data/Pavia_University/paviau_refmap_1D.mat
    
    % the following is for the ELM classifier
    NumberofHiddenNeurons = 500;
    % Type of activation function:
    ActivationFunction = 'sig';

    refmap = double(refmap);
    
    % get the set of training samples
    train_data = x(train(:,1,k), :);
    % and the corresponding labels
    train_label = refmap(train(:,1,k), :);
    % get the set of test sampling
    test_data = x(test(:,k), :);
    % and the corresponding labels
    test_label = refmap(test(:,k), :);


Auxiliary Files
-----------------
The following functions are used for creating the WTSS-EMP scheme presented in
Ref. [1]. 

- cdf97_forward.m and cdf97_inverse.m : Forward and Inverse 1D Wavelet CDF9/7.
    These functions are used for reducing the dimensionality of the hyperspectral 
    image several times and restore the original image. The DWT is implemented
    in dwt97_1D.m, and the inverse DWT in idwt97_1D.m.
    
- dwtDenoising_2DensityNeighShrink.m : It is used for preparing the hyperspectral
    bands of the image 'x' before applying the denoising using neighbouring 
    wavelet coefficients. 
    The 2D Double-Density DWT software used in these scheme is available at
    http://eeweb.poly.edu/iselesni/DoubleSoftware. 
    WAVELET SOFTWARE AT POLYTECHNIC UNIVERSITY, BROOKLYN, NY.

- emp.m : It performs an Extended Morphological Profile (EMP) from the 'n' 
    first components of the image 'f'. The structuring element radius ranges
    from 1 to 7 (step size 2). Therefore, the morphological profile has a size 
    of 2 x 4 + 1, and the EMP a size of (2 x 4 + 1) x number of bands of 'f'.
               
- row_major.m : It transforms a hyperspectral cube in a row-major matrix (one
     pixel-vector per row).

- row_major_restore.m : It transform a row-major maxtrix hyperspectral image in 
    a hyperspectral cube.
    
- sack_vector.m : It is used for creating the stack vector from the 
    hyperspectral image denoised and the EMP.
    
- scale_data.m : It is used for scaling the values of a matrix from a user 
    specified minimum to a user specified maximum.

- README : This file.


License
-----------------
The underlying research materials for this article are licensed under a Creative 
Commons Attribution-NonCommercial-ShareAlike 4.0 International License. 
https://creativecommons.org/licenses/by-nc-sa/4.0/

Contact
-----------------
Pablo Quesada Barriuso <pablo.quesada@usc.es>
Centro Singular de Investigación en Tecnoloxías da Información (CiTIUS)
Universidade de Santiago de Compostela
Rua de Jenaro de la Fuente Dominguez, 15782, Santiago de Compostela, Spain.
http://citius.usc.es/

