function [ z ] = cdf97_forward( x, n )
%
% 1D Wavelet CDF9/7 applied 'n' times to each row of the matrix 'x' 
%
% Authors: Pablo (MATLAB version)
% The Daubechies D4 Wavelet Transform 
% http://www.bearcave.com/misl/misl_tech/wavelets/daubechies/
% INPUT
%   'x' the hyperspectral data with one pixel-vector per row
%   'n' the levels of decomposition
% OUTPUT
%   'z' the 1D-DWT applied to each hyperspectral band n times

if nargin ~= 2
    error('Use: cdf97_forward( x, n )');
else
    
    z = dwt97_1D(x);
    
    N  = size(z,2);
    
    for k=2:n
        N2 = ceil(N/pow2(k-1));
        N2 = N2 + mod(N2,2);    % redondear para ser par
        z(:,1:N2) = dwt97_1D( z(:,1:N2) );
    end;

end;

end
