function [ z ] = cdf97_inverse( x, n )
%
% 1D Wavelet CDF9/7 applied 'n' times to each row of the matrix 'x'
%
% Authors: Pablo (MATLAB version)
% The Daubechies D4 Wavelet Transform 
% http://www.bearcave.com/misl/misl_tech/wavelets/daubechies/
% INPUT
%   'x' the hyperspectral cube
%   'n' the levels of decomposition
% OUTPUT
%   'z' the Inverse 1D-DWT applied to each hyperspectral band n times

if nargin ~= 2
    error('Use: cdf97_inverse( x, n )');
else
   
    z = idwt97_1D( x );
    
    N = size(x,2);
    
    for k=n-1:-1:1
        N2 = ceil(N/pow2(k-1));
        N2 = N2 + mod(N2,2);  % redondear para ser par
        z(:,1:N2) = idwt97_1D( z(:,1:N2) );
    end;
    
end;

end
