function [ w ] = dwt97_1D( a, varargin )
%
% 1D wavelet 9 / 7 applied to each row of the matrix a
%
% Authors: Francisco Arguello (C version) Pablo (MATLAB version)

    M1 = 9;
    M2 = 7;

    h  = [0.0378284555,-0.0238494650,-0.1106244044,0.3774028556,0.8526986790,0.3774028556,-0.1106244044,-0.0238494650,0.0378284555];
    g  = [-0.0645388826,0.0406894176,0.4180922732,-0.7884856164,0.4180922732,0.0406894176,-0.0645388826];

%% ---------------- forming low-pass and high-pass filters --------------- 
% CDF 9/7 filters: decomposition low-pass lp and high-pass hp
%                  reconstruction low-pass lpr and high-pass hpr
% The filter coefficients have several forms.
% What D.S.Taubman et al. suggest are used here:
    % lp = [.026748757411 -.016864118443 -.078223266529 .266864118443];
    % h = [lp .602949018236 fliplr(lp)];
    % hp = [.045635881557 -.028771763114 -.295635881557];
    % g = [hp .557543526229 fliplr(hp)];

    % for readability
    N = size(a,2);

    %% pad input array so the convolution filter fit within the array
    aa = padarray(a, [0 9], 'circular', 'post');
    %aa = padarray(a, [0 9], 'replicate', 'post');
    %aa = padarray(a, [0 9], 'symmetric', 'post');
    %aa = padarray(a, [0 9], 'post');


    % MATLAB tip: The size of the indicated variable or array appears to be 
    % changing with each loop iteration. 
    w = zeros(size(a,1),N);

    % approximation stored in the first middle of the array
    il = 1;             % low pass index
    % detials stored in the second middle of the array
    ih = ceil(N/2) + 1; % high pass index

    %% for each coefficient in the spectral domain...
    for i=1:2:N
        lp=0;
        hp=0;
        
        for j=1:M1
            lp = lp + h(j) * aa(:,i+j-1);  % low pass
        end
        for j=1:M2
            hp = hp + g(j) * aa(:, i+j-1); % high pass
        end
        % update all the points with the corresponding value
        % we put the low pass in the first half of the array...
        w(:,il)   = lp;
        il = il + 1;
        % ... and the high pass in the second half of the array
        w(:,ih) = hp;
        ih = ih + 1;
    end
    % does the user asked for decimation ? return only the half of the
    % array
    if nargin == 2 && strcmp(varargin{1},'low')
        w = w(:,1:ceil(N/2));
    end
end
