function w = dwtDenoising_2DensityNeighShrink( x )
%
% Parameters:
%    @INPUT Hyperspectral image 'x'
%
%    @OUTPUT 'w' 2D wavelet shrinkage to each hyperspectral band using an 
%                universal threshold and neighbouring wavelet coefficients. 

% Selesnik framework is available at http://eeweb.poly.edu/iselesni/DoubleSoftware/ 
% WAVELET SOFTWARE AT POLYTECHNIC UNIVERSITY, BROOKLYN, NY.
addpath './Double-Density Wavelet Software/FrameletCode/'

    % number of levels for 2D wavelet denoising
    L = 4;

    % number of rows
    r = size(x,1);
    % number of columns
    c = size(x,2);
    
    % Rounding up to the nearest multiple of a number
    % http://stackoverflow.com/questions/3407012/c-rounding-up-to-the-nearest-multiple-of-a-number
    roundtor = r;
    reminder = mod( r, pow2(L));   % 16 = 2⁴ as 4 is the number of DWT applied
    if reminder ~= 0
        roundtor = roundtor + pow2(L) - reminder;
    end    
    roundtoc = c;
    reminder = mod( c, pow2(L));   % 16 = 2⁴ as 4 is the number of DWT applied
    if reminder ~= 0
        roundtoc = roundtoc + pow2(L) - reminder;
    end    
    
    % pad inpput for dwt with symmetric values
    x = padarray(x, [roundtor-r roundtoc-c], 'symmetric', 'post');

    % prepare output (also padded!)
    w = zeros(size(x,1), size(x,2), size(x,3));
     
    [af, sf] = filters1;
    % number of level of decomposition
    J = L;
    for i=1:size(x,3)
        x_i = x(:,:,i);
        % forward transform
        wd = double_f2D( x_i, J, af );
       
        % hi{4} = 'hi1 hi1' subband and L es el HH_1 del ultimo nivel
        % hi{8} = 'hi2 hi2' subband and L es el HH_1 del ultimo nivel
        % en wd{4} están las HH1 y HH2 del último nivel de decomposition
        % y en wd{1} las primeras
        H1H1 = wd{L}{4};
        % Noise standard deviation
        sigma1 = median(abs(H1H1(:)))/0.6745;
        H2H2 = wd{L}{8};
        % Noise standard deviation
        sigma2 = median(abs(H2H2(:)))/0.6745;
        
        sigma = (sigma1+sigma2);
        
        % number of pixel of the image
        N = numel(H1H1) * 2;
        % Universal threshold
        T = sigma*sqrt(2*log(N));

        % soft-threshold
        for k=1:J
            % loop through coefficients
            for s = 1:8
                % image denoising using neighbouring wavelet coefficients
                wd{k}{s} = neighShrink2d(wd{k}{s}, T);
            end
        end
        
        wd = double_i2D( wd, J, sf );
        w(:,:,i) = wd;
    end;

     % remove padding from output
     w(r+1:roundtor,:,:) = [];     % rows
     w(:,c+1:roundtoc,:) = [];     % columns
   
end

function [ y ] = neighShrink2d ( w, t )

windowsize  = 3;
windowfilt = ones(1,windowsize);

sij = conv2(windowfilt,windowfilt,w.^2,'same');
% The '+' sign at the end of the formula means to keep the value if it is
% posivite, and set it to zero otherwise.
y = w .* max(0, (1 - (t^2 ./ sij)));

end

