function y = emp( f, n )
%
% Perform an Extended morphological Profile from the 'n' first components
% Structuring element radius ranges from 1 to 7 (step size 2)
%
% Author: Pablo Quesada Barriuso

e = 4;      % number of opening / closing
s = 2;      % step size
m = e*s-1;  % maximum radius size

if nargin ~= 2
    error('Use: EMP( hyperspectral_cube, n )')
else
    y = uint8( zeros( size(f,1), size(f,2), (e + e + 1) * n ) );
    i = 1;
    for j=1:n
        % create closing profile
        for r=m:-s:1
            %% y(:,:,i) = imclosing_by_reconstruct( f(:,:,j), r );
            se = strel('disk', r);
            % it's the dual operation of the opening, therefore we complement the input
            y(:,:,i) = imreconstruct( imcomplement(imdilate(f(:,:,j), se)), imcomplement(f(:,:,j)) );
            % and complement the final solution
            y(:,:,i) = imcomplement( y(:,:,i) );
            i = i+1;
        end;
        % the middle of the profile is the current image
        y(:,:,i) = f(:,:,j);
        i = i + 1;
        % create opening profile
        for r=1:s:m
            %% y(:,:,i) = imopening_by_reconstruct( f(:,:,j), r );
            se = strel('disk', r);
            y(:,:,i) = imreconstruct( imerode(f(:,:,j), se), f(:,:,j) );
            i = i+1;
        end;
    end;

end;

end
